%% Multivariate Probit
% 
%
%  written by: 
%      Aline Talhouk
%      Department of Statistics           
%      University of British Columbia
%      http://www.stat.ubc.ca/~a.talhouk
%
%% 
%  Last modified: August 18, 2011
%
%  Disclaimer:  No guarantee that this code is free of bugs.
%               If you find any mistakes please email a.talhouk@stat.ubc.ca
%%

function ret= rtnorm(lower, upper)
% rtnorm  Generates a random observation from normal distribution truncated
% to the interval [lower, upper] with mean=0, and sd=1
% Implementation of the Algorithm from Robert (1995).
%
% Aline Talhouk, UBC (2011)
%
% Reference: Robert, C. (1995), Simulation of Truncated Normal Variables,
%            Statistics and Computing, 5, 121-125.


if (lower>upper)
    alg=1;
else
         if  ((lower < 0 & upper ==Inf) | (lower == -Inf & upper > 0) | (isfinite(lower) & isfinite(upper) & (lower < 0) & (upper > 0) & (upper - lower > sqrt(2 * pi))))        alg=2;
    else
        if (lower >= 0 & (upper > lower + 2 * sqrt(exp(1))/(lower + sqrt(lower^2 + 4)) * exp((lower * 2 - lower * sqrt(lower^2 + 4))/4)))
            alg=3;
        else
            if upper <= 0 & (-lower > -upper + 2 * sqrt(exp(1))/(-upper + sqrt(upper^2 + 4)) * exp((upper * 2 - -upper * sqrt(upper^2 +  4))/4))
                alg=4;
            else
                alg=5;
            end
        end
    end
end
done=0;

switch alg
    case 1
        error('Lower must be less than upper'); 
    
    case 2% Regular Rejection
        while ~done
            y = randn(1);
            done= (y >= lower& y <= upper);
            if done
                ret=y;
            end
        end
    
    case 3 %Proposition 2.3
        while ~done
            a = (lower + sqrt(lower^2 + 4))/2;
            z = exprnd(a) + lower;
            u = rand(1);
            done = (u <= exp(-(z - a)^2/2)) & (z <= upper);
            if done
                ret=z;
            end
        end

    case 4 %opposite direction of 2
        while ~done
            a = (-upper + sqrt(upper^2 + 4))/2;
            z = exprnd( a) - upper;
                u = rand(1);
            done = (u <= exp(-(z - a)^2/2)) & (z <= -lower);
            if done
                ret=-z;
            end
        end
    case 5
        while ~done
            z = rand(1)*(upper-lower)+lower;
            if lower>0
                rho = exp((lower^2 - z^2)/2);
            else
                if upper < 0
                    rho=exp((upper^2 - z^2)/2);
                else
                    rho=exp(-z^2/2);
                end
            end
            u = rand(1);
            done = (u <= rho);
            if done
                ret=z;
            end
        end

end