function [edge_ok, C]=check_edge_add_same_component_cell(a,b, jtree, sepsize, cliques)
% inputs: 1. a, b, the indicies of the edge vertices with respect
%              to the original ordering in the adjacency matrix g
%           2. jtree, the adjacency matrix of a junction tree
%              with respect to cliques.
%           3. sepsize, a matrix array of the size of the separator sets, in which
%              sepsize(i,j) = [number of elements in intersection between
%              cliques cliques{i} and cliques{j} if they are adjacent, and zero else.
%           4. cliques, a 1 x t cell array of a perfect sequence of
%              (nonempty) cliques,
%              such as from chordal_to_ripcliques_cell.m
% outputs: 1. edge_ok=1/0
%              (yes/no, can add and remain chordal)
%          2. C, the new clique which contains the vertices of the added edge,
%              as represented by a 1 x |C| array of vertex indicies
%              with respect to original g
%              The likelihood depends on C, so it is outputted by
%              this routine.
%%% NOTE: this routine is ONLY called by the main program if
%%% the verticies are in the same connected component.
edge_ok=0;
quit = 0;
fork=0;
   CASE_same_branch=0;
   CASE_sats_on_b2_branch =0;
   locate_a2=0;
   t=size(cliques,2);    % END 1st SECTION for code description
   index_b=find_clique_containing_cell(b, cliques);
   index_a=find_clique_containing_cell(a, cliques);
%%% Re_name verticies so b is in clique{index_b2}, a is in clique{index_a2}
   index_b2=max([index_a,index_b]);
       % index_b2 >1, as it is greater of the two, and
       % since a,b is NOT an edge, can't have index_a2=index_b2
   if index_b2==index_b; b2=b; a2=a;
     else b2=a; a2=b;
   end
   clear index_a;


index=index_b2;
next_index=parents_vertex_cell(jtree, index);
      % tree, so parent is a single vertex, not a vector
for dummy=1: index_b2-1;
     % perform the loop b2-1 times if no break
    if   isempty(next_index)==1 & locate_a2 ==0;
        CASE_same_branch=0; break,
     end
      % if next_index is empty AND locate_a2 ==0,
      % then at the top of the tree and KNOW
      % that index_a2 and index_b2 are on separate branches.
      % BUT can't assume the fork is top of tree.
      % Could be case of 3-2-4-5 with 1-2 the top of tree.
    if is_in(a2, cliques{next_index}) ==1 & isempty(next_index) ==0;
         index_a2=next_index; locate_a2= 1; CASE_same_branch=1; break,
    end
         % if you find a2 before you get to the top of the tree,
         % then KNOW that index_a2 and index_b2 are on same
         % branch of tree. So exit the loop, and set
         % locate_a2 case indicator.
     index=next_index;
     next_index=parents_vertex_cell(jtree, index);
    % if a2 is in any clique on the same side of the root
    % vertex in jtree, then that clique and the first clique
    % in the RIP ordering containing b2 will be the
    % end points of the shortest path between 2 containing
    % cliques for the verticies of the edge considered.
    % Performs the loop at least b2-1 times if no break,
    % which is the longest possible path to the top of a
    % connected component of the possibly disconnected tree.
end;    % END 2nd SECTION for code description
if CASE_same_branch==0,
    index_a2=find_clique_containing_cell(a2, cliques);
end
    % IF CASE_same_branch==0, (so locate_a2==0) then
    % must be fork between them. So index_a2 for shortest path
    % is first clique in RIP ordering containing a2.
intersection_of_cliques=intersect(cliques{index_a2}, cliques{index_b2});
s=length(intersection_of_cliques);
    % find intersection, so can test to see if
    % it is a separator. In practice, only need to
    % check that the size of this intersection is
                                          

        % equal to the size of a separator on this path.
    if s==0; edge_ok=0; C=[]; quit=1; end
        % the empty set is not a separator in a connected
        % tree, so condition cannot be satisfied. exit at this stage.
    if jtree(index_a2, index_b2)==1;
        edge_ok=1; C=union( [a,b], intersection_of_cliques); quit=1;
    end    % END 3rd SECTION for code description
                 % if the cliques are adjacent in the tree, their
                 % intersection is by definition a separator so exit.
%%%  if quit still zero, next test all the separators
%%%  between clique{index_a2} and clique{index_b2}
%%%  at any stage, if there exists a separator of length s
%%%  then know edge legal, so set edge_ok=1, quit=1, and exit loop.
    if ( (quit ==0) & (CASE_same_branch==1) );
          % first consider where a2 is on same branch as b2. only need to
          % test path from b2 to a2
        bottom=index_b2;
        next_parent_b2=parents_vertex_cell(jtree, index_b2);
        while ((next_parent_b2 >= index_a2) )
           if sepsize(next_parent_b2, bottom) == s;
                edge_ok=1; C=union( [a,b], intersection_of_cliques);
                quit=1; break,
            end,
                   % only need to test equality of size,
                   % since intersection contained in every intermediate
                   % clique by RIP and junction tree property
          bottom=next_parent_b2;
          next_parent_b2 = parents_vertex_cell(jtree, bottom);
      end; % END 4th SECTION for code description
    elseif ( (quit==0) & (CASE_same_branch==0) );
        % if on different branches, have to test from index_b2 and
        % index_a2 to fork between them. CANNOT go to edge beyond fork.
        % Safest strategy is to find the fork.
          bottom_a2=index_a2; next_parent_a2=parents_vertex_cell(jtree, index_a2);
          bottom_b2=index_b2; next_parent_b2=parents_vertex_cell(jtree, index_b2);
          ancestors_a2=zeros(1, index_a2); ancestors_b2=zeros(1, index_b2);
          next_ancestor_a2=index_a2; next_ancestor_b2=index_b2;
          for count=1:index_a2
              ancestors_a2(count)=next_ancestor_a2;
              next_ancestor_a2 =parents_vertex_cell(jtree, next_ancestor_a2);
              if isempty(next_ancestor_a2), break, end
          end
                                                         
      ancestors_a2=nonzeros( ancestors_a2)';
               % convert to only the ancestors using existing matlab function.
      for count=1:index_b2
          ancestors_b2(count)=next_ancestor_b2;
          next_ancestor_b2 =parents_vertex_cell(jtree, next_ancestor_b2);
          if isempty(next_ancestor_b2), break, end
      end
      ancestors_b2=nonzeros(ancestors_b2)';
                % convert to only the ancestors using existing matlab function.
      fork_set=intersect(ancestors_a2, ancestors_b2);
      fork=max(fork_set);
                % the clique where the two paths up the tree meet must be the
                % highest index in the intersection to the root.
      % END 5th SECTION for code description
      num_seps_branch_b2=length(find(ancestors_b2>fork));
    for count=1: num_seps_branch_b2
          index_row=ancestors_b2(count+1); index_col=ancestors_b2(count);
             % sepsize is lower diagonal zero, and the ancestors are stored
             % in descending order [index_b2,..., 1]
         if sepsize( index_row, index_col) == s;
               edge_ok=1; C=union( [a,b], intersection_of_cliques);
               quit=1; CASE_sats_on_b2_branch=1; break,
          end,
     end
    if (quit==0 & CASE_sats_on_b2_branch==0)
          % don't want to perform the above loop
          % if found separator=intersection.
        num_seps_branch_a2=length(find(ancestors_a2>fork));
        for count =1:num_seps_branch_a2
            index_row=ancestors_a2(count+1); index_col=ancestors_a2(count);
                % sepsize is lower diagonal zero, and the ancestors are stored
                % in descending order [index_a2,..., 1]
            if sepsize( index_row, index_col) == s;
                 edge_ok=1; C=union( [a,b], intersection_of_cliques);
                 quit=1; break,
            end,
        end
    end
end
if quit==0; edge_ok=0; C=[]; end
% END 6th and final SECTION for code description

