function [edge_ok, C]=check_edge_add_same_component_zo(a,b, jtree, sepsize, cliques)
% inputs: 1. a, b, the indicies of the edge vertices with respect
% to the original ordering in the adjacency matrix g
% 2. jtree, the adjacency matrix of a junction tree
% (with respect to cliques) from ripcliques_to_jtree_zo.
% 3. sepsize, a matrix array of the size of the
% separator sets, in which
% sepsize(i,j) = |C_i intersection C_j| between
% adjacent cliques C_i and C_j with respect to jtree.
% 4. cliques, a p x p matrix array respresentation of a
% perfect sequence of cliques of g.
% (eg, from chordal_to_ripcliques_zo.m)
% outputs: 1. edge_ok=0(no)/1(yes)
% 2. C, the p x 1 zero/one vector representation of the new clique
% which contains the vertices of the added edge,
% The likelihood depends on C, so it is outputted by
% this routine.
%%% NOTE: this routine is ONLY called by the main program if
%%% the verticies are in the same connected component.


edge_ok=0;
quit = 0;
fork=0;
CASE_same_branch=0;
CASE_sats_on_b2_branch =0;
locate_a2=0;
p=size(cliques,1);
% END 1st SECTION for code description
ab_edge_vec=zeros(p,1);
ab_edge_vec(a)=1;
ab_edge_vec(b)=1;
index_b=find_clique_containing_zo(b, cliques);
index_a=find_clique_containing_zo(a, cliques);
%%% Re_name verticies so b is in cliques(index_b2, :), a is in cliques(index_a2, :);
%%% Note: jtree indicies correspond to verticies of cliques in RIP,
%%% i.e. order of vertex=clique is same as column index of cliques.
index_b2=max([index_a,index_b]);
if index_b2==index_b;
    b2=b;
    a2=a;
else
    b2=a;
    a2=b;
end
% index_b2 >1, since max[ ]>1 (Can't have both a and b in first clique, as
% there is no edge between them. Also, |index_a -index_b|>=1.
% hence if..else ensures correct ordering of index_a2, index_b2
% i.e. know clique(index_a2, :) precedes index_b2

clear index_a;
index=index_b2;
next_index=find(parents_vertex_zo(jtree, index));
% tree, so parent is unique

for dummy=1: index_b2-1;

    %this performs the loop at least b2-1 times if no break
    if sum(next_index)==0 & locate_a2 ==0;
        % if parents empty (=[0 0 ... 0]') and not found a2, must be at tree top
        CASE_same_branch=0; break,
    end
    %  if next_index is empty AND locate_a2 ==0, then at tree top and KNOW index_a2
    %  and index_b2 are on different branches. BUT can't assume the fork is top of tree.
    %  could be case of 3-2-4-5 with 1-2 the top of tree.


    clique_next_index=cliques(:, next_index);
    if clique_next_index(a2,1) ==1 & sum(next_index) ~=0;
        % if a2 is in clique_next_index, and not at tree top
        index_a2=next_index;
        locate_a2=1;
        CASE_same_branch=1;
        break,
    end
    % if you find a2 before you get to the top of tree, then

    % KNOW index_a2 and index_b2 are on same branch of tree. So
    % break and set
    % locate_a2 case indicator.
    index=next_index;
    next_index=find(parents_vertex_zo(jtree, index));
end;
% if a2 is in any clique on the same side of the root vertex in jtree,
% then that clique and the first clique in the RIP ordering containing b2
% will be the end points of the shortest path between 2 containing cliques
% for the verticies of the edge considered. Performs the loop at least b2-1
% times if no break, which is the longest possible path to the top of a
% connected component of the possibly disconnected tree.
% END 2nd SECTION for code description

if CASE_same_branch==0,
    index_a2=find_clique_containing_zo(a2, cliques);
end
% IF CASE_same_branch==0, (so locate_a2==0) then
% must be fork between them. So index_a2 for shortest path
% is first clique in RIP ordering containing a2.
clique_a2_int_clique_b2=intersect_zo(cliques(:,index_a2), cliques(:,index_b2));
s=sum(clique_a2_int_clique_b2);
% find intersection, so can test to see if
% it is a separator. In practice, only need to
% check that the size of this intersection is
% equal to the size of a separator on this path.
if s==0;
    edge_ok=0;
    C=zeros(p,1);
    quit=1;
end
% the empty set is not a separator in a connected
% tree, so condition cannot be satisfied. exit at this stage.

if jtree(index_a2, index_b2)==1;
    edge_ok=1;
    C=union_zo(ab_edge_vec, clique_a2_int_clique_b2);
    quit=1;
end
% END 3rd SECTION for code description
% if the cliques are adjacent in the tree, their
% intersection is by definition a separator so exit.
%%%if quit still zero, next test all the separators between
% clique_a2 and clique_b2 at any stage,
% if there exists a separator of length s then know edge add legal,
% so set edge_ok=1, quit=1, and exit loop.

if ( (quit ==0) & (CASE_same_branch==1) );

    % first consider where a2 is on same branch as b2. only need to
    % test path from b2 to a2
    bottom=index_b2;
    next_parent_b2=find(parents_vertex_zo(jtree, index_b2));
    while (sum(next_parent_b2)>0 ) & (next_parent_b2 >= index_a2 )
        if sepsize(next_parent_b2, bottom) == s;
            edge_ok=1;
            C=union_zo(ab_edge_vec, clique_a2_int_clique_b2);
            quit=1;
            break,
        end,

        % only need to test equality of size,
        % since intersection contained in every intermediate
        % clique by RIP and junction tree property
        bottom=next_parent_b2;
        next_parent_b2=find(parents_vertex_zo(jtree, bottom));
    end;
    % END 4th SECTION for code description
elseif ( (quit==0) & (CASE_same_branch==0) );
    % if on different branches, have to test from index_b2 and
    % index_a2 to fork between them. CANNOT go to edge beyond fork.
    % Safest strategy is to find the fork.
    %%% NOTE: in below, the parent clique vertex of is unique,
    %%% as only 1 parent in trees
    next_parent_a2=find(parents_vertex_zo(jtree, index_a2));
    next_parent_b2=find(parents_vertex_zo(jtree, index_b2));
    ancestors_a2=zeros(1, index_a2); ancestors_b2=zeros(1, index_b2);
    % ancestors_a2 elements are actual number indicies of clique ancestors
    ancestors_a2_col_vec=zeros(p,1);
    ancestors_b2_col_vec=zeros(p,1);
    % the col_vec equivalent
    next_ancestor_a2=index_a2;
    next_ancestor_b2=index_b2;
    for count=1:index_a2 ancestors_a2(count)=next_ancestor_a2;
        ancestors_a2_col_vec(next_ancestor_a2)=1;
        next_ancestor_a2 =find(parents_vertex_zo(jtree, next_ancestor_a2));
        if sum(next_ancestor_a2)==0,
            break,
        end
    end
    for count=1:index_b2 ancestors_b2(count)=next_ancestor_b2;
        ancestors_b2_col_vec(next_ancestor_b2,1)=1;
        next_ancestor_b2 =find(parents_vertex_zo(jtree, next_ancestor_b2));
        if sum(next_ancestor_b2)==0,
            break,
        end
    end
    fork_set=intersect_zo(ancestors_a2_col_vec, ancestors_b2_col_vec);
    fork=max(find((fork_set)));
    % need to find the place in the tree where the fork is, as defined


    % by the clique which is at this point of intersection the clique where the
    %two paths up the tree meet must be the biggest index in the intersection to
    % the root. i.e. if they both share clique(:,3), then the path of ancestors
    % for both must include the ancestors of clique(:,3) which if were clique2,
    %clique1, then where the paths split is at clique 3 = max(find(fork_set)))

    % END 5th SECTION for code description
    num_seps_branch_b2=length(find(ancestors_b2>fork));
    %%%% VERY IMPORTANT %%% only test separators between clique_fork and indicies

    for count=1: num_seps_branch_b2 index_row=ancestors_b2(count);
        index_col=ancestors_b2(count+1);
        % sepsize is lower diagonal zero, and the ancestors are stored
        % in descending order [index_b2,..., 1]
        if sepsize( index_row, index_col) == s;
            edge_ok=1;
            C=union_zo( ab_edge_vec, clique_a2_int_clique_b2);
            quit=1;
            CASE_sats_on_b2_branch=1;
            break,
        end,
    end
    if (quit==0 & CASE_sats_on_b2_branch==0)
        % don't want to perform the above loop
        % if found separator=intersection.
        num_seps_branch_a2=length(find(ancestors_a2>fork));
        for count =1:num_seps_branch_a2 index_row=ancestors_a2(count+1);
            index_col=ancestors_a2(count);
            % sepsize is lower diagonal zero, and the ancestors are stored
            % in descending order [index_a2,..., 1]
            if sepsize( index_row, index_col) == s;
                edge_ok=1;
                C=union_zo( ab_edge_vec, clique_a2_int_clique_b2);
                quit=1;
                break,
            end,
        end
    end
end

if quit==0
    edge_ok=0;
    C=[];
end
% END 6th and final SECTION for code description

