function [cliques]=chordal_to_ripcliques_zo(g, order) 
% inputs: 1. g, the p x p symmetric adjacency matrix with 
%            respect to an original ordering v_1, ..., v_p 
%         2. order, the output of check_chordal or any other 
%            perfect numbering of the vertex indicies. 
% output: 1. cliques, a p x p matrix representation of the 
%            cliques of g in perfect order. 
%            i.e. each column cliques(:,i) is the ith ordered clique 
%            in a sequence of cliques that satisfies the running intersection property. 
%            If the jth row of cliques(:,i) equals one, then v_j is an 
%            element of clique C_i. 
%
%
% Note: the maximum number of cliques for any graph is p, 
% which occurs when the v_i are all independent so each clique comprises 
% a single variable vertex. 

p=size(g,1);
pa=zeros(p,p); 
num_pa=zeros(1,p); 
% initialise the vector of number of predecessors 

for i=2:p;
    v=order(i);
    pre_v=zeros(p,1);
    pre_v(order(1:i-1))=1; 
    % create 0/1 col vector representing predecessors of v 
    ns=neighbours_vertex_zo(g, v); 
    % find set of neighbours of each v=order(i) 
    pa_i=intersect_zo(ns, pre_v);
    num_pa(i)=size(find(pa_i),1); 
    pa(:, v)=pa_i; 
    % find the sets of those neighbours which precede 
    % v=order(i) with respect to order. Store answer for cliques. 
    % so if order=[1 3 7 5 2 4 6], pa(:,order(4)=5)=[0 0 1 0 0 0 1]' 
    % pa = 
    % 0 0 1 0 0 0 1 
    % 0 0 0 1 0 0 0 
    % 0 0 0 0 1 0 1 
    % 0 0 0 0 0 0 0 
    % 0 0 0 0 0 0 0 
    % 0 0 0 0 0 0 0 
    % 0 1 0 0 1 1 0 
    % num_pa ordered as per order (i) = 0 1 2 2 1 1 1 
    % corresponding to v= 1 3 7 5 2 4 6
    %
    % 
    %
    % eg num of pre nbs of 3 for ladder test=sum(:,3), etc. 
    % note that the ith column corresponds to the pre-nbs of the ith vertex in order; 
    % i.e v=order(i), and NOT v=i. Simly for cardinality
    % Need to keep variables ordered as per the % mcs ordering, the vector
    % called order. 
end; 
ladder=zeros(1,p); 
% NOTE: for ladder(i)~=0, ladder(i)=order(i) 
for i=1:p; 
    if i==p | sum(pa (:, order(i))) >= sum(pa(:,order(i+1))) 
        %if i=p or cardinality of pa decreasing with 
        % v=order(i), then the vertex v=order(i) is a ladder vertex. 
        % eg, i=4, order as above has ladder [0 0 7 5 2 4 6] 
        % as == or decrease occurs num_pa(i), i=3 4 5 6 7 
        % and order(3)=7,order(4)=5,order(5)=2,order(6)=4,order(7)=6. 
        ladder(i)=order(i); 
        % make this v the next ladder vertex 
    end; 

end; 
        %%%%%%%%%%%%%%%%%%%%%%%
        %% NOTE: i output non-empty columns first, so clique ordering follows 
        %% matrix cliques column ordering. 
        %% if only 3 cliques, then cliques(:, 3:n)=zero(n,1); 
        %% cliques(:,i)=union of the ith non-zero ladder vertex and its predecessors) 
        %% eg cliques(:,3)=[0 1 0 0 0 0 0]+[0 0 0 0 0 0 1] = {2} U {7} 
        %% NOTE: for ladder(i)~=0, ladder(i)=order(i) 
        
        cliques=zeros(p, p); 
        % only create clique columns if it's non-empty set 
        index_non_zero_ladder=0;
        for i=1:p; 
            if ladder(i)~=0; v_col_i=zeros(p,1); 
                % create column vector rep. of v=ladder(i) 
                v_col_i(ladder(i),1)=1; 
                % ladder(i)=order(i) 
                index_non_zero_ladder=index_non_zero_ladder+1; 
                cliques(:, index_non_zero_ladder)=union_zo(v_col_i,pa(:,ladder(i))); 
            end; 
        end;
        
        % Theorem: cliques(index_non_zero_ladder, 1:size_cliques) are the cliques of g 
        % and clique ordering satisfies the RIP.