function [B_hat, K_hat]=g_constrain_zo(B, order, cliques)
% g must be decomposable
% B, symmetric positive definite
% inputs: 1. B, the p x p symmetric matrix with
% respect to an original ordering v_1, ..., v_p of g
% 2. cliques, a p x p matrix array respresentation of a
% perfect sequence of cliques of g.
% (eg, from chordal_to_ripcliques_zo.m)
% output: 1. B_hat, the g-constrained version of B
% 2. K_hat, the g-constrained version of inv(B_hat)
% THEORY: if (i,j) is edge or i=j, B_hat(i,j)=B(i,j)
% else B_hat satisfies [inv(B_hat)](i,j)=0
% such a B_hat for decomposable models has been
% shown to be unique
% (see either Grone et al 84, or
% Speed and Kiiveri, 1986, p. 142 Thereom 1.
% Calculate using pp. 144, 145 Lauritzen 96, based on
% Lemma 5.5 p.136 Lauritzen 96 for block sum expression.
p=size(B,1);
seps=seps_resids_hists_zo(cliques);
num_cliques=max(find( sum(cliques,1)~=0));
% sum(cliques,1)) gives sum of each col=numelts in clique
% find..~=0 returns indicies of non-zero cols, so take max for num
sum_big_K_cliques=zeros(p); sum_big_K_seps=zeros(p);
for j=1:num_cliques
    clear cj B_cj K_cj
    cj=find(cliques(:,j))'; % note the transpose =cliques{j};
    B_cj=B(cj, cj);
    K_cj=inv(B_cj);
    big_Kj=zeros(p);
    big_Kj([cj], [cj])=K_cj;
    sum_big_K_cliques=sum_big_K_cliques+big_Kj;
end
for j=1:num_cliques %%% NOTE i always set the first empty, as everyone numbers as 2, num_cliques.
    clear sj B_sj K_sj
    sj=find(seps(:,j))'; %=seps{j};
    B_sj=B(sj, sj);
    K_sj=inv(B_sj);
    big_Kj=zeros(p);
    big_Kj([sj], [sj])=K_sj;
    sum_big_K_seps=sum_big_K_seps+big_Kj;
end
K_hat=sum_big_K_cliques-sum_big_K_seps;
B_hat=inv(K_hat);