function [sigma_id, K_id, sigma_id_rev_perf, K_rev_perf]=generate_HIW_g_delta_identity_zo(g, cliques, delta)
% inputs: 1. g, the p x p symmetric matrix with
% respect to an original ordering v_1, ..., v_p
% 2. cliques, a p x p matrix array representation of the cliques of g.
% such as from chordal_to_ripcliques_zo.m
% output: 1. sigma_id, a random draw from HIW(g, delta, identity)
% 2. K_id=inv(sigma_id), a random draw from HW(g, delta, identity)
% THEORY: Roverato00 Theorem 3.
% GENERATE Sigma and inv(Sigma)=K, from g-conditional HIW(g, delta, identity):
% i.e. a Sigma for empty graph
% NOTE K_id has EXACT zeros in the right places, no rounding errors
% First step to generate a random draw=Sigma_i~HIW(g_i, delta, Phi_i),
% where Phi_i are iterate
% outputs of the mcmc for each graph iterate g_i.
index_finish=0; index_start=0;
randn('seed',sum(100*clock));%added Aline
num_cliques=max(find( sum(cliques,1)~=0));
[seps, residuals, histories]=seps_resids_hists_zo(cliques);
% NOTE the seps(:,1) will be zeros(p,1), because everyone writes them as S_2,...
num_Rj=zeros(1, num_cliques);
%%% create perfect ordering as per C1, R2, R3,..definition
p=length(g);
perfect_order=zeros(1,p);
c_1=find(cliques(:,1))';
perfect_order(1:length(c_1))=c_1;
index_finish=length(c_1);
for j=2:num_cliques
Rj=find(residuals(:,j))'; % note the transpose
num_Rj(1,j)=length(Rj);
index_start=index_finish+1;
index_finish=index_start+num_Rj(j)-1;
perfect_order(index_start:index_finish)=Rj;
end
% now reverse the ordering, and use rev_perf=opposite
% order to perfect_order
% for indexing the graph and the parameter Phi.
%% NOTE: can NOT assume that the ordering of g_reverse=
% my mcs ordering of g (had i computed it) in reverse. But
% by construction, it is opposite to a perfect order which
% satisfies at least one mcs with respect to g, and is constructed
% as C1, R2, R2, .., Rk as req'd by Roverato00p100
rev_perf=zeros(1,p);
index=0;
for i=1:p
rev_perf(i)=perfect_order(p-index);
index=index+1;
end
g_rev_perf=zeros(p);
g_rev_perf=g(rev_perf,rev_perf);
%%%% to go back and forth: note that
% perf_order=p-rev_perf+1; perf+reverse=p+1
%%%%%%%%%%%
Psi=zeros(p); % Psi is wrt reverse perfect ordering
% Psi(i,i) is a random Chi_squared(delta+nu_i).
for i=1:p
if i==p,
Psi(p,p)=(chi2rnd(delta))^(.5);
else
% find nu_i=num of parents=adj. predecessors
% of each vertex i wrt PERFECT
% order, NOT reverse= an elimination order.
% nu_p=0 because Psi is wrt reverse ordering.
% nu_i, i>1 could be zero for disconnected case.
clear nu_i
nu_i=sum(g_rev_perf(i,i+1:p));
Psi(i,i)=(chi2rnd(delta+nu_i))^(.5);
end
end
% Psi(i,j), j >i is random N(0,1) if an edge
% i,j exists in g, and zero otherwise.
for i=1:p-1
for j=i+1:p
if (g_rev_perf(i,j)==0)
Psi(i,j)=0;
else
Psi(i,j)=randn;
end
end
end
K_rev_perf=zeros(p); sigma_id_rev_perf=zeros(p);
K_rev_perf=Psi'*Psi;
sigma_id_rev_perf=inv(K_rev_perf); % is HIW(g_rev_perf, delta, id_rev_perf)
% Note that this is covariance of g wrt the rev_perf ordering,
% NOT wrt g. Should have zeros where g_rev_perf does, less diag
%%% Now need to do inverse permutation to get back to the Sigma for g
inverse_permute=zeros(1,p);
for j=1:p
inverse_permute(j)=find(rev_perf==j);
end
K_id=zeros(p); sigma_id=zeros(p);
K_id=K_rev_perf(inverse_permute, inverse_permute);
sigma_id=sigma_id_rev_perf(inverse_permute, inverse_permute); % ~HIW(g, delta, id)
% Check that sigma_id has zeros in right place:
%inv_sigma_id=inv(sigma_id);
% indices=find(~(inv_sigma_id==0)); inv_sigma_id(indices)=1;
% note that indices is not pairs, but where (1,1)=1, (1,2)=2, etc