%% Multivariate Probit
% 
%
%  Adapted from Helen Armstrong code
%
%% 
%  Last modified: August 18, 2011
%
%  Disclaimer:  No guarantee that this code is free of bugs.
%               If you find any mistakes please email a.talhouk@stat.ubc.ca
%%
function [g_next accept_percent_sample]=...
    generate_graph_and_analysis_zo(g_current, delta, Phi, N, S_N, warmup, iter_multiple, iters_to_keep)
% inputs: 1. g_current, adj. matrix of current graph iterate
% 2., 3. delta, Phi, the parameters of the HIW(g, delta, Phi) prior for the covariance
% 3. N, the number of observations
% 4. S_N, the sum of squares matrix of the data
% 5. warmup, the number of warmup iterations
% 6. iter_multiple, the factor of iters_to_keep for the total number of iterates
% 7. iters_to_keep, the number of graph iterates to record
% outputs: 1. g_next, the last graph iterate
%          2. graph_pm, a p x p adjacency matrix of the average graph sampled
%          3. graph_tally, a pxpx num_diff_graphs array of the distinct
%           adjacency matrices.
%          4. graph_tally_counts, a 1 x p vector of counts of each graph sampled.
p=length(Phi);
iter=iter_multiple*iters_to_keep;
if (iter <= warmup),
    error('not enough iterations specified');
end

sample=iter-warmup;
counter_for_keeping=0;
graph_cumulative=zeros(p,p);
graph_pm=zeros(p,p);
graph_iter=zeros(p,p, iters_to_keep);
graph_tally=zeros(p,p, 1);
num_of_diff_graphs=1;
accept_count_warmup=0; accept_count_sample=0;
accept_percent_warmup=0; accept_percent_sample=0;
g_next=g_current;
for i=1:iter;
    clear cliques jtree order
    clear sepsize reach_graph
    clear g_proposal g_i
    equal_indicator=0;
    g_current=g_next;
    [chordal, order]=check_chordal(g_current) ;%Aline added
    cliques=chordal_to_ripcliques_zo(g_current,order) ; % find cliques
    jtree=ripcliques_to_jtree_zo(cliques); % create jtree
    sepsize=sepsize_zo(cliques, jtree); % find size of separators
    reach_graph=reachability_graph(g_current); % find path matrix
    u=rand; accept_prob=0;
    [g_proposal, a,b, C, CASE_add, CASE_delete]=...
        next_graph_candidate_zo(g_current, jtree, sepsize, cliques, reach_graph);
    if CASE_add==1,
        ln_likelihood_ratio=ln_h_ratio_zo( C, a, b, delta, Phi)...
            -ln_h_ratio_zo( C, a, b, delta+N, Phi+S_N);
        likelihood_ratio=exp(ln_likelihood_ratio);
    elseif CASE_delete==1,
        ln_likelihood_ratio=ln_h_ratio_zo( C, a, b, delta+N, Phi+S_N)...
            -ln_h_ratio_zo( C, a, b, delta, Phi);
        likelihood_ratio=exp(ln_likelihood_ratio);
    end
    accept_prob=min(1, likelihood_ratio);
    % accept g_proposal with probability=accept.
    if u<= accept_prob,
        g_next=g_proposal;
        if i<=warmup
            accept_count_warmup=accept_count_warmup+1;
        else accept_count_sample=accept_count_sample+1;
        end
    else
        g_next=g_current;
    end
    accept_percent_sample=accept_count_sample/iters_to_keep;
end

   