function [ln_h]=ln_h_ratio_zo( C, a, b, delta, Phi)
% inputs: 1. C, the p x 1 zero/one vector representation of the new clique
% which in the case of edge addition
% contains the vertices of the added edge, or
% in the case of edge deletion, contained the deleted edge.
% 2. a, b, the index of the edge vertices with respect to the
% original ordering in the adjacency matrix g.
% 3. delta, integer > 0
% 4. Phi>0, p x p parameter for Sigma_E~HIW(g, delta, Phi)
% output: 1. log of h(g, delta, Phi)/h(g, delta, Phi), the
% ratio of normalising constants for
% p(Sigma_E) where E is set of edges in g, and
% p(Sigma_E) where E is set of edges in g.
% marginal likelihood ratio is h_ratio(delta, Phi)/h_ratio(delta+n, Phi+S_N)
% (the 2*pi terms cancel out)
C_nodes=(find(C))'; % NOTE THE TRANSPOSE
D=[a,b];
% C_nodes and D are the actual nodes, not
% the col_vec representation
Sq2=setdiff(C_nodes,D);
numSq2=length(Sq2);
delta_star=delta -1;
Phi_CC=zeros(length(C_nodes));
Phi_CC=[ Phi(Sq2, Sq2) Phi(Sq2,D) ;...
    Phi(D, Sq2) Phi(D,D)];
L=(chol(Phi_CC))' ;
L_DD=L([numSq2+1, numSq2+2], [numSq2+1, numSq2+2]);
% this is correct: DD is the LOWER block.
% Sq2 is index of upper block.
% Hence the terms of the matrix containing DD
% are indexed as numSq2+1, numSq2+2
l_aa=L_DD(1,1);
l_bb=L_DD(2,2);
l_ab=L_DD(2,1); % L is lower diag: L_DD(1,2)=0
ln_top=(delta_star + numSq2 +2)*(log(l_aa )+log( l_bb));
ln_bottom= (delta_star + numSq2 +1)/2*...
    (log(l_aa^2*l_ab^2 + l_aa^2*l_bb^2)) ;
ln_gam_bit=-log(2*sqrt(pi))+gammaln(( delta_star + numSq2 +1)/2 )...
    - gammaln(( delta_star + numSq2 +2)/2 );
ln_h= ln_top-ln_bottom + ln_gam_bit;