function [g_proposal, a,b, C, CASE_add, CASE_delete]=...
next_graph_candidate_cell(g_current, jtree, sepsize, cliques, reach_graph)
% inputs: 1. g_current, the adjacency matrix of the current graph in the chain.
% 2. jtree, the adjacency matrix of a junction tree with respect
% to cliques.
% 3. sepsize, a matrix array of the size of the separator sets, in which
% sepsize(i,j) = [number of elements in intersection between
% cliques cliques{i} and cliques{j} if they are adjacent, and zero else.
% 4. cliques, a 1 x t cell array of nonempty cliques
% of a chordal graph in RIP ordering
% (such as re_index_cliques from chordal_to_ripcliques_cell.m).
% 5. reach_graph, a p x p symmetric matrix, in which
% reach_graph(i,j) = 1 iff there is a path from v_i to v_j in g
% outputs:1. g_proposal, the adjacency matrix of the graph proposed for
% the next graph in the chain. g_proposal will only be
% accepted as g_next if it passes the MH acceptance test.
% 2., 3. a, b, the indicies of the vertices of
% the edge whose addition or deletion from g_current gives g_proposal.
% 4. C, the clique in g_proposal which contains the extra edge
% in the case of addition, or the clique which contains the edge
% in g_current which is proposed for deletion.
% Likelihood depends on C, so it is outputted by this routine.
% 5., 6. CASE_add, CASE_delete, the 0/1 indicators of whether
% an edge is proposed for deletion or addition respectively.
p=length(g_current);
g_proposal=g_current;
% initialise the proposal graph to the current graph
CASE_add=0; CASE_delete=0;
while isequal(g_proposal,g_current);
edge=next_edge_candidate(g_current);
i=edge(1); j=edge(2);
if g_current(i,j)==1,
[CASE_delete, C_potential_delete]=check_edge_delete_cell(i,j, cliques);
if ( CASE_delete==1) ;
g_proposal(i,j)=0; g_proposal(j,i)=0;
CASE_delete=1; a=i; b=j; C=C_potential_delete;
end
elseif ( g_current(i,j)==0 & reach_graph(i,j)==0 );
CASE_add=1; a=i; b=j; C=[a,b];
% if adding edge between 2 disjoint trees, new clique
% including that edge must be comprised soley of edge nodes a, b
g_proposal(i,j)=1; g_proposal(j,i)=1;
% if v_i and v_j are in different connected components of a decomposable
% graph, then adding (v_i, v_j) is legal.
elseif ( g_current(i,j)==0 & reach_graph(i,j) ==1 ),
% v_i, v_j must be in the same connected component of tree for check_edge_add.
[CASE_add, C_potential_add]=...
check_edge_add_same_component_cell(i,j, jtree, sepsize, cliques);
if CASE_add==1
a=i; b=j; C=C_potential_add;
g_proposal(i,j)=1; g_proposal(j,i)=1;
end
end
end; % external while