function reach_graph = reachability_graph(g) 
% inputs: 1. g, the p x p symmetric adjacency matrix with 
%           respect to an original ordering v_1, ..., v_p 
% output: 1. reach_graph, a p x p symmetric matrix, in which 
%        reach_graph(i,j) = 1 iff there is a path from i to j 
%
% This computes g = g + g^2 + ... + g^{p-1}, which is the transition matrix showing 
% all possible vertices that can be reached in 1, ..., p-1 steps. Note that the longest 
% possible path is p-1, so only need to consider up to and including paths of length p-1

p = size(g,1); 
A = g; 
reach_graph = zeros(p); 
for i=1:p-1 
    reach_graph = reach_graph + A; 
    A = A * g; 
end 
reach_graph = (reach_graph > 0); 
% C = (C > 0) gives a matrix with 1s in every place where C > 0