function [jtree]=ripcliques_to_jtree_zo(cliques) 
% input: 1. cliques, a p x p matrix array respresentation of a 
%           perfect sequence of cliques of g. 
%           (eg, from chordal_to_ripcliques_zo.m)
% output: jtree, the adjacency matrix of a junction tree 
%           (not necessarily unique) with respect to cliques 
% WARNING: this algorithm only works for cliques outputted 
% with respect to a maximum cardinality search ordering of 
% the cliques, as then the cliques are already ordered with
% respect to the rank of the highest vertex in the clique. 
%For example, if mcs ordering of vertices is [6 5 3 1 4 2], 
%then a clique comprised of {6,1} will precede a clique 
%comprised of {4,2} since 6 and 1 precede 4 and 2 in the ordering [6 5 3 1 4 2]. 
p=size(cliques, 1); 
clique_sizes=zeros(1,p); 
clique_sizes=sum(cliques, 1); 
% this is a 1 x p vector of column totals. 
% summing the column cliques(:,i) gives the number of vertices 
% in clique C_i. 
num_cliques=size(find(clique_sizes), 2); 
score=zeros(1,p); 
jtree=zeros(p,p); 
for i=2:num_cliques; 
    for k=1:i-1; 
        score(k)=sum(intersect_zo(cliques(:,i), cliques(:,k)) ); 
    end; 
    if max(score)~=0 ; 
        % only add the edge if clique i IS connected to one of its 
        % predecessors. if score is all zeros, then clique has no intersection 
        % with any of its predecessors. Since the cliques are in RIP, it must 
        % follow that we are no longer in the same connected component of 
        % the graph. 
        % if clique i has no intersection with any of the preceding cliques,
        % then the graph is disconnected, so the adjacency matrix will have 
        % a zero row/column for this i, and we have a forest, not a j_tree. 
        j=argmax(score); 
        jtree(i,j)=1; 
        jtree(j,i)=1;
    end; 
end;

