function [seps, resids, hists]=seps_resids_hists_zo(cliques) 
% input: 1. cliques, a p x p matrix array respresentation of a 
% perfect sequence of cliques of g. 
% output:  1. seps, a p x p matrix array of the separators wrt the
%          ordering of the matrix array cliques. Note that by definition,
%          the indexes j of S_j begin at 2, so below defines the first separator 
%          as the empty set. 
%          2. resids, a p x p matrix array of the histories wrt 
%          the ordering of the matrix array cliques. Note that by definition, 
%          the indexes j of R_j begin at 2, so below defines the first residual as the empty set. 
%          3. hists, a a p x p matrix array of the separators wrt the ordering of the matrix array cliques.

p=size(cliques,1); 
num_cliques=max(find( sum(cliques,1)~=0)); 
% sum(cliques,1)) gives sum of each col=number of elts in clique 
%find...~=0 returns indicies of nonzero cols, so take max for num 

seps=zeros(p, p); 
resids=zeros(p, p); 
% NOTE i always set the first as empty, so index=2,..., num_cliques. 
hists=zeros(p, p); 
hists(:,1)=cliques(:,1); 
for index=2:num_cliques; 
    hists(:,index)=union_zo(cliques(:,index), hists(:,index-1)); 
    seps(:,index)=intersect_zo(cliques(:,index), hists(:,index-1)); 
    resids(:,index)=setdiff_zo(cliques(:,index), hists(:,index-1)); 
end; 
% Sj are intersection of total HISTORY and the % new clique, not Cj and
% Cj-1