%% Multivariate Probit
% 
%
%  written by: 
%      Aline Talhouk
%      Department of Statistics           
%      University of British Columbia
%      http://www.stat.ubc.ca/~a.talhouk
%
%% 
%  Last modified: August 18, 2011
%
%  Disclaimer:  No guarantee that this code is free of bugs.
%               If you find any mistakes please email a.talhouk@stat.ubc.ca
%%

function [betahat M] = sampleBETA(Sig,w,X,Xi)
% sampleBETA generates samples from the posterior distribution of beta
% conditional on the other variables
%
% Input:
% Sig: A sample from the posterior of the corresponding covariance in the
%      expanded space.
% w : The latent Gaussian data in the expanded space (w=zD)
% X : The n x p design matrix 
% Xi: updated posterior parameter Xi = inv(X'X+psi)
%
% Output
% betahat: A sample from the posterior of beta
% M: Xi*X'W

J=length(Sig);
    [n p]=size(X);
    L=kron(Sig,Xi);
    M=Xi*X'*w;
    B=reshape(M,p*J,1);
    betahat=reshape(mvg(B,L,1),p,J);
    
end
