%% Multivariate Probit
% 
%
%  written by: 
%      Aline Talhouk
%      Department of Statistics           
%      University of British Columbia
%      http://www.stat.ubc.ca/~a.talhouk
%
%% 
%  Last modified: August 18, 2011
%
%  Disclaimer:  No guarantee that this code is free of bugs.
%               If you find any mistakes please email a.talhouk@stat.ubc.ca
%%

function [R Rinv sig Dinv]= sampleR(w,iXi,M)
% sampleR generates samples from the posterior distribution of a
% saturated correlation matrix conditional on the other variables
%
% Input:
% w : The latent Gaussian data in the expanded space (w=zD)
% iXi and M are updated posterior parameters for beta iXi = X'X+psi and M =Xi*X'W
%
% Output
% R: A sample from the posterior of R
% Rinv: inv(R) to avoid computing it again
% sig: A sample from the posterior of the corresponding covariance in the
%      expanded space
% Dinv: A diagonal matrix of inverse standard deviations corresponding to
%       sig
%
[n J]=size(w);

S =(w'*w)+eye(J)-M'*iXi*M;

v=(n+J+1)+J+1; 

[sig icov]=invwishirnd(S,v);
D=diag(sqrt(diag(sig)));
Dinv=inv(D);
R=Dinv*sig*Dinv;
Rinv=D*icov*D;
