%% Multivariate Probit
% 
%
%  written by: 
%      Aline Talhouk
%      Department of Statistics           
%      University of British Columbia
%      http://www.stat.ubc.ca/~a.talhouk
%
%% 
%  Last modified: August 18, 2011
%
%  Disclaimer:  No guarantee that this code is free of bugs.
%               If you find any mistakes please email a.talhouk@stat.ubc.ca
%%

function [X y Y Xb] = simprobit(R,beta,n) 
%  simprobit generates random Multivariate Binary Data by truncating Multivariate
%  Gaussians at 0.
%
%  Inputs: 
%  R a symmetric JxJ covariance matrix
%  beta a pxJ regression coefficients
%  n the number of observations
%
%  Output:
%  X a design matrix
%  y binary data
%  Y continuous Gaussian data
%  Xb X*beta

[p,J]=size(beta);

X=rand(n,p)-0.5;
Xb=X*beta;
eps=mvnrnd(zeros(1,J),R,n);

Y=Xb+eps;
y=(Y>0);
y=y+1;
      

