%% Multivariate Probit
%
%
%  written by:
%      Aline Talhouk
%      Department of Statistics
%      University of British Columbia
%      http://www.stat.ubc.ca/~a.talhouk
%
%%
%  Last modified: August 18, 2011
%
%  Disclaimer:  No guarantee that this code is free of bugs.
%               If you find any mistakes please email a.talhouk@stat.ubc.ca
%%
clear all
close all

load('sixdat.mat')
[n J]=size(y);

smoke=reshape((X(:,2,:)==1), J,n)';
smkndx=(smoke(:,1)==1);
X=[ones(n,1) smkndx];

N=100000;
sat=0;
y=y+1;
[Csamples iCsamples Bsamples]= mvprobit(y,X,N,sat,'SixRes');

%%%%
clear all
load('SixRes.mat')
[n J]=size(y);
p=size(X,2);
smoke=X(:,2);
smkndx=(smoke==1);

%Summary statistics

mean(y(smkndx,:)-1,1) %Percentage of kids with attacks at the different ages born to smokers 
mean(y(~smkndx,:)-1,1) %Percentage of kids with attacks at the different ages born to non-smokers 
mean(y-1,1)
sum(y(smkndx,:)-1,1)
sum(y(~smkndx,:)-1,1)
sum(y-1,1)
mean(y-1,1)

%Posterior Analysis of the collected samples
[graph_tally graph_tally_counts graph_pm graph_iter]= graphtallyMC(iCsamples,J);
poste=normalize(graph_tally_counts);%MC posterior probability of each graph

[a in]=sort(poste,'descend');
mostpopgraphs=graph_tally(:,:,in);

%Plot of 4 most popular graph with its posterior probability (Figure 6 in
%paper)
figure;
for i=1:4
subplot (2,2,i);
plotg4(mostpopgraphs(:,:,i))
title(sprintf('%s%6.2f','Post= ',a(i)))
end
suptitle('Most Probable Structures')

%Matrix of Edge Marginals
figure;
hintonDiagram(graph_pm); axis('square')
title('Matrix of Edge Marginals')

%Posterior of beta
beta=[-1.118; -0.079 ; 0.152; 0.039];
mysummary(Bsamples)
betaCG=[-0.9851 -1.0321 -1.0595 -1.2419
    0.0065 0.2204 0.1718 0.1554] 
    
%Posterior of R
R=vect2mat([0.5944;0.5278;0.5692;0.6970;0.5633;0.6413],4);
figure; 
mysummary(Csamples)

%Table 4

T1= sum(sum(y==1,2)==4)/537 %Never experienced wheezing
T2= sum(sum(y==2,2)==4)/537 %Always wheezing
T3= (sum((y(:,1)==1)& sum(y(:,2:end)==2,2)==3)+sum(sum(y(:,1:2)==1,2)==2& sum(y(:,3:end)==2,2)==2))/537


%Model Checking: Posterior Predictive Checks
ind=randperm(length(Bsamples));
L=25000;
for j=1:L
i=ind(j);
b=Bsamples(i,:);
s=vect2mat(Csamples(i,:),J); graph_iter(:,:,i);
si=vect2mat(iCsamples(i,:),J); 
Xbhat=reshape(X*reshape(b,2,J),J,n)';

%Generating the data
%Method 1: truncating Gaussians
zrep1=mvnrnd(Xbhat,s);
yrep1=(zrep1>0)+1;
avg(j,:)=mean(yrep1-1);
acc(j,:)=mean(yrep1==y);
Trep(j,1)=sum(sum(yrep1==1,2)==4)/537;
Trep(j,2)=sum(sum(yrep1==2,2)==4)/537;
Trep(j,3)=(sum((yrep1(:,1)==1)& sum(yrep1(:,2:end)==2,2)==3)+sum(sum(yrep1(:,1:2)==1,2)==2& sum(yrep1(:,3:end)==2,2)==2))/537;
end
 q1=quantile(Trep(:,1),[0.025 0.975])*100
 pval1=sum(Trep(:,1)>=T1)/L
 q2=quantile(Trep(:,2),[0.025 0.975])*100
  pval2=sum(Trep(:,2)>=T2)/L
 q3=quantile(Trep(:,3),[0.025 0.975])*100
   pval3=sum(Trep(:,3)>=T3)/L
   

