%% Multivariate Probit
%
%
%  written by:
%      Aline Talhouk
%      Department of Statistics
%      University of British Columbia
%      http://www.stat.ubc.ca/~a.talhouk
%
%%
%  Last modified: August 18, 2011
%
%  Disclaimer:  No guarantee that this code is free of bugs.
%               If you find any mistakes please email a.talhouk@stat.ubc.ca
%%

clear all
close all

n=500; %Number of data points

R=[1 0 0 -0.491 0
    0 1 -0.296 0 0.116
    0 -0.296 1 0 -0.392
    -0.491 0 0 1 0
    0 0.116 -0.392 0 1]

inv(R)
J=length(R);
beta=repmat([-1; 2], 1, J);

N=50000; %Number of MCMC gibbs iterations

for s=1:50 % Repeat experiment on 50 data sets generated with the same parameters
    [X y Y Xb] = simprobit(R,beta,n); %Generate Data from the Probit model
    fname=sprintf('%s%d%s','TrialBinCovar',s,'.mat')
    itime1=cputime;[Csamples1 iCsamples1 Bsamples1]=mvprobit(y,X,N,1,fname);ctime1=cputime;tMC1=ctime1-itime1;%Run with The saturated Model
    itime2=cputime;[Csamples2 iCsamples2 Bsamples2]=mvprobit(y,X,N,0,fname);ctime2=cputime;tMC2=ctime2-itime2;%Run with Model Selection
    save(fname)
end


%%% Produce Figure 1 and 2 from the paper
clear all
close all
load('TrialBinCovar50')
iCsamples=iCsamples2;

% To plot the true structure for comparison
G =[  0     0     0     1     0
    0     0     1     0     0
    0     1     0     0     1
    1     0     0     0     0
    0     0     1     0     0]

postanalysis

%To produce figure 3

clear all
  
 for c=1:50
     fname=sprintf('%s%d%s','TrialBinCovar',c,'.mat')
       load(fname)
       AvE1(c) = CompLoss(Csamples1,R)
       AvE2(c)= CompLoss(Csamples2,R)
 end 
 A=[AvE1; AvE2];
 boxplot(A','label',{'Full Model','Model Selection'})
 title('Entropy Loss')
 
 [h p ci]=ttest(AvE1, AvE2)